@@ -1,7 +1,7 @@
 from simpler._version import __version__
 from simpler.algorithms import DynamicProgramming
 from simpler.bioinformatics import codon_table, monoisotopic_mass_table, monoisotopic_mass_water, parse_fasta, dna_to_rna, rna_to_dna, rna_to_protein, reverse_complement
-from simpler.connectors import MySQL, Excel
+from simpler.connectors import SQL, Excel
 from simpler.files import cwd, load, save, disk_cache, size, find_hidden_compressed, tvshow_rename, directory_compare
 from simpler.format import human_bytes, human_seconds, human_date, random_string, print_matrix, safe_filename
 from simpler.mail import compose, send
