@@ -4,11 +4,12 @@ from re import compile, IGNORECASE
 from simpler.terminal import cprint
 from typing import Any, Generator, List, Optional, Union
 
-def simpler_converter():
+def _mysql_converter():
+	''' Simpler MySQL converter that returns some bytes as strings and decimals as floats. '''
 	from mysql.connector.constants import FieldFlag
-	from mysql.connector.conversion import MySQLConverter
+	from mysql.connector.conversion import MySQLConverter as NativeConverter
 
-	class Res(MySQLConverter):
+	class MySQLConverter(NativeConverter):
 		def row_to_python(self, row, fields):
 			res = super().row_to_python(row, fields)
 			return res
@@ -28,76 +29,109 @@ def simpler_converter():
 			return float(value)
 
 		def _STRING_to_python(self, value, dsc=None):
-			res = super(Res, self)._STRING_to_python(value, dsc)
+			res = super(MySQLConverter, self)._STRING_to_python(value, dsc)
 			if dsc[7] & FieldFlag.BINARY:
 				return res.decode(self.charset)
 			return res
 		_VAR_STRING_to_python = _STRING_to_python
 
-	return Res
+	return MySQLConverter
 
-class MySQL:
-	''' Connector for a mysql backend with a handful of helpers. '''
+class SQL:
+	''' Connector for SQL databases with a handful of helpers. '''
+	ENGINES = 'mysql', 'mssql'
 	def __init__(
 		self, host: str = 'localhost', user: str = 'root', password: str = None, db: str = None,
 		charset: str = 'utf8mb4', collation: str = 'utf8mb4_general_ci', use_unicode: bool = True,
-		max_insertions: int = None, print_queries: bool = False, native_types: bool = True
+		max_insertions: int = None, print_queries: bool = False, native_types: bool = True,
+		engine: str = 'mysql'
 	) -> None:
-		self.max_insertions, self._cursor = max_insertions, None
-		self._connection = {
-			'host': host,
-			'user': user,
+		assert engine in SQL.ENGINES, 'Accepted engine values are: %s.' % ', '.join(SQL.ENGINES)
+		self.max_insertions, self.native_types, self.engine, self.print_queries = max_insertions, native_types, engine, print_queries
+		self._connection, self._cursor, self._initialized = {'user': user}, {}, False
+		if engine == 'mysql':
+			self._init_mysql(charset, collation, host, use_unicode, password, db)
+		elif engine == 'mssql':
+			self._init_mssql(host, password, db)
+	
+	def _init_mysql(self, charset, collation, host, use_unicode, password, db):
+		self._connection.update({
 			'charset': charset,
 			'collation': collation,
-			'use_unicode': use_unicode,
-		}
-		if native_types:
-			self._connection['converter_class'] = simpler_converter()
+			'host': host,
+			'use_unicode': use_unicode
+		})
 		if password:
-			self._connection.update({'passwd': password, 'auth_plugin': 'mysql_native_password'})
+			self._connection.update({
+				'passwd': password,
+				'auth_plugin': 'mysql_native_password'
+			})
 		if db:
 			self._connection['db'] = db
-		self.print_queries = print_queries
-
-	def __del__(self):
-		self.close()
+		self._cursor['buffered'] = True
+	
+	def _init_mssql(self, host, password, db):
+		self._connection['server'] = host
+		if password:
+			self._connection['password'] = password
+		if db:
+			self._connection['database'] = db
 
 	def close(self) -> None:
 		''' Closes the current cursor and connection. '''
-		if self._cursor is not None:
+		if self._initialized:
 			self._cursor.close()
 			self._connection.close()
-			self._cursor = self._connection = None
+			self._initialized = False
+
+	__del__ = close
 
 	def cursor(self):
 		''' Returns the open cursor and initializes the connection if required. '''
-		if self._cursor is None:
-			from mysql.connector import connect
+		if not self._initialized:
+			if self.engine == 'mysql':
+				try:
+					from mysql.connector import connect
+				except ModuleNotFoundError:
+					raise ModuleNotFoundError('Missing MySQL connector. Install a mysql client and then do `pip install mysql.connector`.')
+				if self.native_types:
+					self._connection['converter_class'] = _mysql_converter()
+			else:
+				try:
+					from pymssql import connect
+				except ModuleNotFoundError:
+					raise ModuleNotFoundError('Missing MS-SQL connector. Install a MS-SQL client and then do `pip install pymssql`.')
 			self._connection = connect(**self._connection)
-			self._cursor = self._connection.cursor(buffered=True)
+			self._cursor = self._connection.cursor(**self._cursor)
+			self._initialized = True
 		return self._cursor
 
-	def execute(self, query: str, params: tuple = None, multi: bool = False):
+	def execute(self, query: str, params: tuple = None, multi: bool = False, commit: bool = False):
 		''' Wrapper for the mysql.connector execute method that won't send the params argument
 		if the params are empty, thus avoiding the need to replace % with %%. '''
 		if self.print_queries:
 			self.print_query(query, params)
-		statement = self.cursor().execute(query, params if params is not None and len(params) else None, multi)
-		if multi:
-			try:
-				list(statement)
-			except RuntimeError:  # see https://bugs.mysql.com/bug.php?id=87818
-				pass
-		self._connection.commit()
+		if self.engine == 'mysql':
+			statement = self.cursor().execute(query, params if params is not None and len(params) else None, multi)
+			if multi:
+				try:
+					list(statement)
+				except RuntimeError:  # see https://bugs.mysql.com/bug.php?id=87818
+					pass
+		else:
+			statement = self.cursor().execute(*([query] + ([params] if params else [])))
+		if commit:
+			self._connection.commit()
 
 	def print_query(self, query: str, params: tuple = None, color: str = 'yellow', max_size: int = 1000):
+		''' Shows a query attempting to inject the parameters, for debugging purposes. '''
 		if len(query) > max_size:
 			query = query[:max_size // 2] + '...' + query[-max_size // 2:]
 		try:
 			formatted = query.strip() % params if params is not None and len(params) else query.strip()
 		except:
 			formatted = query.strip()
-		cprint(formatted + ';', fg='yellow')
+		cprint(formatted + ';', fg=color)
 
 	def select(
 		self, table: str, filters: dict = lambda: {}, first_row: bool = False,
@@ -168,17 +202,17 @@ class MySQL:
 
 	def insert(self, query: str, *params: tuple) -> int:
 		''' Inserts a row and returns its id. '''
-		self.execute(query, params)
+		self.execute(query, params, commit=True)
 		return int(self.cursor().lastrowid)
 
-	def insert_all(self, table: str, rows: list, tuple_rows: bool = True) -> Optional[int]:
+	def insert_all(self, table: str, rows: Union[List[dict], List[tuple]], tuple_rows: bool = True, commit: bool = True) -> Optional[int]:
 		''' Insert a list of rows and returns the id of the last one. By default,
 		these rows are a list of {column: value} dicts, but they can be inserted
 		from tuples of values setting `tuple_rows` to True. '''
 		if not len(rows): return
 		while self.max_insertions is not None and self.max_insertions < len(rows):
 			part, rows = rows[:self.max_insertions], rows[self.max_insertions:]
-			self.insert_all(table, part, tuple_rows)
+			self.insert_all(table, part, tuple_rows, commit=False)
 		if tuple_rows:
 			query = 'INSERT INTO %s VALUES %s' % (
 				table,
@@ -194,12 +228,12 @@ class MySQL:
 				','.join(values for _ in rows)
 			)
 			params = [insertion[key] for insertion in rows for key in keys]
-		self.execute(query, params)
+		self.execute(query, params, commit=commit)
 		return int(self.cursor().lastrowid)
 
 	def apply(self, query: str, *params: tuple) -> int:
 		''' Applies a modification (update or delete) and returns the number of affected rows. '''
-		self.execute(query, params)
+		self.execute(query, params, commit=True)
 		return int(self.cursor().rowcount)
 
 	def update(self, table: str, updates: dict = lambda: {}, filters: dict = lambda: {}) -> int:
@@ -221,7 +255,7 @@ class MySQL:
 				values.append(k + '=%s ')
 				params.append(v)
 			query += 'WHERE ' + ' AND '.join(values)
-		self.execute(query, params)
+		self.execute(query, params, commit=True)
 		return int(self.cursor().rowcount)
 
 	def escape(self, value: Any, is_literal: bool = True) -> str:
